import type { LanguageInput, ThemedToken, ThemeRegistrationAny, TokensResult } from '@shikijs/types'
import type { FunctionComponent, ReactNode } from 'react'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useMemo, useState } from 'react'
import { Platform, ScrollView, Text, View } from 'react-native'

const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface SyntaxHighlighterProps {
  text: string
  language?: LanguageInput
  languageId?: string
  theme: ThemeRegistrationAny
  fontSize?: number
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const engine = createJavaScriptRegexEngine({ forgiving: true, simulation: true, cache: new Map() })

const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const stylesheet = useMemo(() => {
    const processedStyles = new Map<string, { color?: string, fontStyle?: string }>()

    if (!theme.tokenColors) {
      console.warn('Theme does not contain tokenColors')
      return processedStyles
    }

    theme.tokenColors.forEach(({ scope, settings }) => {
      if (!scope || !settings)
        return
      const scopes = Array.isArray(scope) ? scope : [scope]
      scopes.forEach((singleScope) => {
        processedStyles.set(singleScope, {
          color: settings.foreground,
          fontStyle: settings.fontStyle,
        })
      })
    })

    return processedStyles
  }, [theme])

  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    let mounted = true

    async function fetchTokens() {
      if (!text) {
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighterCore({ langs: language ? [language] : [], themes: [theme], engine })
        const result = highlighter.codeToTokens(text, { lang: languageId || 'plaintext', theme, includeExplanation: true }) as TokensResult

        if (mounted) {
          setTokens(result.tokens || [])
          setError(null)
        }
      }
      catch (err) {
        console.error('Syntax highlighting failed:', err)
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'))
          setTokens([[{ content: text, color: '#d8dee9', offset: 0 }]])
        }
      }
    }

    fetchTokens()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  const renderTokenLine = (tokens: ThemedToken[], lineIndex: number): ReactNode => (
    <View key={lineIndex} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {tokens.map((token, tokenIndex) => {
        const firstScope = token.explanation?.[0]?.scopes[0]
        const applicableStyle = firstScope ? stylesheet.get(firstScope.scopeName) : {}

        return (
          <Text
            key={`${lineIndex}-${tokenIndex}`}
            style={[
              { fontFamily: FONT_FAMILY, fontSize, color: applicableStyle?.color },
              applicableStyle?.fontStyle === 'italic' ? { fontStyle: 'italic' } : {},
              applicableStyle?.fontStyle === 'bold' ? { fontWeight: 'bold' } : {},
              textStyle || {}, // Ensure textStyle is always an object
            ]}
          >
            {token.content}
          </Text>
        )
      })}
    </View>
  )

  if (error) {
    return (
      <ScrollView horizontal {...scrollViewProps} style={[{ flex: 1, minHeight: 20 }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 16 }, containerStyle]}>
        <Text style={[{ fontFamily: FONT_FAMILY, fontSize }, textStyle]}>{text}</Text>
      </ScrollView>
    )
  }

  return (
    <ScrollView horizontal {...scrollViewProps} style={[{ flex: 1, minHeight: 20 }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 16 }, containerStyle]}>
      <View>{tokens.map((line, index) => renderTokenLine(line, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
